@@ -37,6 +37,7 @@ STAILQ_HEAD(sst_order_list, sst_element);
 struct sst_element {
     uint32_t index;
     char *string;
+    uint8_t is_rich_string;
 
     STAILQ_ENTRY (sst_element) sst_order_pointers;
     RB_ENTRY (sst_element) sst_tree_pointers;
@@ -64,7 +65,8 @@ extern "C" {
 
 lxw_sst *lxw_sst_new();
 void lxw_sst_free(lxw_sst *sst);
-struct sst_element *lxw_get_sst_index(lxw_sst *sst, const char *string);
+struct sst_element *lxw_get_sst_index(lxw_sst *sst, const char *string,
+                                      uint8_t is_rich_string);
 void lxw_sst_assemble_xml_file(lxw_sst *self);
 
 /* Declarations required for unit testing. */
