@@ -140,6 +140,15 @@ lxw_xml_data_element(FILE * xmlfile,
     fprintf(xmlfile, "</%s>", tag);
 }
 
+/*
+ * Write an XML <si> element for rich strings, without encoding.
+ */
+void
+lxw_xml_rich_si_element(FILE * xmlfile, const char *string)
+{
+    fprintf(xmlfile, "<si>%s</si>", string);
+}
+
 /*
  * Escape XML characters in attributes.
  */
@@ -153,19 +162,19 @@ _escape_attributes(struct xml_attribute *attribute)
     while (*p_attr) {
         switch (*p_attr) {
             case '&':
-                strncat(p_encoded, LXW_AMP, sizeof(LXW_AMP) - 1);
+                memcpy(p_encoded, LXW_AMP, sizeof(LXW_AMP) - 1);
                 p_encoded += sizeof(LXW_AMP) - 1;
                 break;
             case '<':
-                strncat(p_encoded, LXW_LT, sizeof(LXW_LT) - 1);
+                memcpy(p_encoded, LXW_LT, sizeof(LXW_LT) - 1);
                 p_encoded += sizeof(LXW_LT) - 1;
                 break;
             case '>':
-                strncat(p_encoded, LXW_GT, sizeof(LXW_GT) - 1);
+                memcpy(p_encoded, LXW_GT, sizeof(LXW_GT) - 1);
                 p_encoded += sizeof(LXW_GT) - 1;
                 break;
             case '"':
-                strncat(p_encoded, LXW_QUOT, sizeof(LXW_QUOT) - 1);
+                memcpy(p_encoded, LXW_QUOT, sizeof(LXW_QUOT) - 1);
                 p_encoded += sizeof(LXW_QUOT) - 1;
                 break;
             default:
@@ -195,15 +204,15 @@ lxw_escape_data(const char *data)
     while (*data) {
         switch (*data) {
             case '&':
-                strncat(p_encoded, LXW_AMP, sizeof(LXW_AMP) - 1);
+                memcpy(p_encoded, LXW_AMP, sizeof(LXW_AMP) - 1);
                 p_encoded += sizeof(LXW_AMP) - 1;
                 break;
             case '<':
-                strncat(p_encoded, LXW_LT, sizeof(LXW_LT) - 1);
+                memcpy(p_encoded, LXW_LT, sizeof(LXW_LT) - 1);
                 p_encoded += sizeof(LXW_LT) - 1;
                 break;
             case '>':
-                strncat(p_encoded, LXW_GT, sizeof(LXW_GT) - 1);
+                memcpy(p_encoded, LXW_GT, sizeof(LXW_GT) - 1);
                 p_encoded += sizeof(LXW_GT) - 1;
                 break;
             default:
